@@ -12,7 +12,7 @@ Dancer::Plugin::Database - easy database connections for Dancer applications
 
 =cut
 
-our $VERSION = '1.42';
+our $VERSION = '1.50';
 
 my $settings = undef;
 
@@ -65,7 +65,7 @@ register database => sub {
     $handle = $handles{$pid_tid}{$handle_key} || {};
     
     if ($handle->{dbh}) {
-        if ($conn_details->{connection_check_threshold} &&
+        if ($handle->{dbh}{Active} && $conn_details->{connection_check_threshold} &&
             time - $handle->{last_connection_check}
             < $conn_details->{connection_check_threshold}) 
         {
@@ -78,7 +78,7 @@ register database => sub {
                 Dancer::Logger::debug(
                     "Database connection went away, reconnecting"
                 );
-                if ($handle->{dbh}) { $handle->{dbh}->disconnect; }
+                if ($handle->{dbh}) { eval { $handle->{dbh}->disconnect } }
                 return $handle->{dbh}= _get_connection($conn_details);
 
             }
@@ -192,7 +192,7 @@ sub _get_connection {
 sub _check_connection {
     my $dbh = shift;
     return unless $dbh;
-    if (my $result = $dbh->ping) {
+    if ($dbh->{Active} && (my $result = $dbh->ping)) {
         if (int($result)) {
             # DB driver itself claims all is OK, trust it:
             return 1;
@@ -491,6 +491,9 @@ Alberto Simões
 
 James Aitken (LoonyPandora)
 
+Mark Allen (mrallen1)
+
+
 =head1 BUGS
 
 Please report any bugs or feature requests to C<bug-dancer-plugin-database at rt.cpan.org>, or through
